#!/usr/bin/env python
import io
import sys
import time
import random
import logging
from sched import scheduler
from threading import Thread
from argparse import ArgumentParser

from flask import Flask, render_template, Response
from matplotlib.figure import Figure
from matplotlib.backends.backend_agg import FigureCanvasAgg as FigureCanvas


# setup logging
logging.basicConfig(
    format='%(asctime)s - %(levelname)s - %(filename)s(%(lineno)d) - %(message)s',
    stream=sys.stdout,
    level=logging.DEBUG
)

# setup random generator
random.seed()


class SensorThread(Thread):
    '''a thread to collect sensor data'''
    def __init__(self):
        '''initialize daemon thread'''
        super().__init__()
        self.daemon = True
        self.sched = scheduler(time.time, time.sleep)
        
    def run(self):
        '''run thread'''
        while True:
            self.sched.enter(5, 5, self.read_sensors)
            self.sched.run()

    def read_sensors(self):
        '''DUMMY: dummy function to read sensors'''
        temperature = random.uniform(-10.0, 40.0)
        humidity = random.uniform(0.0, 100.0)
        windspeed = random.uniform(0.0, 50.0)
        winddirection = random.uniform(0, 360.0)
        logging.info(f'humidity: {humidity}, temperature: {humidity}, '
                     f'windspeed: {windspeed}, winddirection: {winddirection}')

        
# setup flask application
app = Flask(__name__)

# flask view functions
@app.route('/')
def index():
    return render_template('index.html')

@app.route('/temperatures')
def temperatures():
    return render_template('temperatures.html')

@app.route('/temperatures_image')
def temperatures_image():
    '''DUMMY: regenerate random data plot'''
    fig = Figure((16, 7)) # image size in inches
    axis = fig.add_subplot(1, 1, 1)
    xs = range(25)
    ys = [random.randint(-10, 40) for x in xs]
    axis.plot(xs, ys)
    output = io.BytesIO()
    FigureCanvas(fig).print_png(output)
    return Response(output.getvalue(), mimetype='image/png')


# main function
def main():    
    # parse command-line arguments
    parser = ArgumentParser(description='run a background task to read and write pins')
    parser.add_argument('--host', '-H', default='127.0.0.1', help='ip address to listen, default 127.0.0.1')
    parser.add_argument('--port', '-P', type=int, default=5000, help='port to listen, default 5000')
    parser.add_argument('--debug', '-d', action='store_true', help='enable debugging, default false')
    args = parser.parse_args()

    # if running in debug mode, show warning
    if args.debug:
        import click
        click.secho(' * WARNING: in DEBUG mode, SensorThread.read_sensors() will be called twice!', fg='red')

    # start sensor reader
    SensorThread().start()

    # run flask app
    app.run(host=args.host, port=args.port, debug=args.debug)

    
if __name__ == '__main__':
    main()
